#ifndef __Mt_Constraint_H__
#define __Mt_Constraint_H__

#include "../../common/common.h"

namespace MaskTools { namespace Constraints {

typedef enum {
   CPU_NONE    = 0,
   CPU_MMX     = 0x01,
   CPU_ISSE    = 0x02,
   CPU_SSE2    = 0x04,
   CPU_SSE3    = 0x08,
   CPU_3DNOW   = 0x10,
   CPU_3DNOW2  = 0x20,

} CpuFlags;

typedef enum {
   MODULO_NONE = 0,
   MODULO_2    = 1,
   MODULO_4    = 3,
   MODULO_8    = 7,
   MODULO_16   = 15,
   MODULO_32   = 31,

} Modulo;

Modulo toModulo(int n);

typedef enum {
   ALIGNMENT_NONE = 0,
   ALIGNMENT_2    = 1,
   ALIGNMENT_4    = 3,
   ALIGNMENT_8    = 7,
   ALIGNMENT_16   = 15,
   ALIGNMENT_32   = 31,

} Alignment;

Alignment toAlignment(int n);

class Constraint {
   CpuFlags    flags;
   Modulo      moduloX;
   Modulo      moduloY;
   Alignment   alignment;
public:
   Constraint();
   Constraint(CpuFlags flags, Modulo moduloX, Modulo moduloY, Alignment alignment);
   Constraint(CpuFlags flags, int nWidth, int nHeight, const Byte *pAddress, int nPitch);
   Constraint(CpuFlags flags, int nWidth, int nHeight, int nAddress, int nPitch);
   bool respect(const Constraint &constraint) const;
};

template<class P>
class Processor {
   P           *processor;
   int         nSpeed;
   Constraint  constraint;
public:
   Processor() : processor(NULL), nSpeed(-1) { }
   Processor(P *processor, const Constraint &constraint, int nSpeed) : processor(processor), constraint(constraint), nSpeed(nSpeed) { }
   bool respect( const Constraint &constraint) const { return this->constraint.respect(constraint); }
   int speed() const { return nSpeed; }
   operator P*() const { return processor; }
};

template<class P>
class ProcessorList : public std::list<Processor<P> >
{
public:
   P *best_processor(const Constraint &constraint) const 
   {
      int nBestSpeed = -1;
      P *best_proc = NULL;
      for ( ProcessorList<P>::const_iterator it = begin(); it != end(); it++ )
      {
         if ( it->speed() > nBestSpeed && it->respect(constraint) )
         {
            nBestSpeed = it->speed();
            best_proc = *it;
         }
      }

      return best_proc;
   }
};

} }

#endif
