#include "params.h"

namespace MaskTools { namespace Parameters {

/* Operator */
Operator::Operator()             : mode(NONE)   , nValue(-1)      { }
Operator::Operator(Mode mode)    : mode(mode)   , nValue(-1)      { }
Operator::Operator(Byte nValue)  : mode(MEMSET) , nValue(nValue)  { }
Operator::Operator(int nValue)   : mode(MEMSET) , nValue(nValue)  { }

bool  Operator::operator ==(const Operator &o)  const { return mode == o.mode; }
bool  Operator::operator ==(Mode mode)          const { return mode == this->mode; }
Operator::operator Mode()                       const { return mode; }
int   Operator::value()                         const { return nValue; }

/* Value */
Value::Value()                   : type(UNDEFINED) , defined(false) { }
Value::Value(Type t)             : type(t)         , defined(false) { }
Value::Value(int n)              : type(INT)       , defined(true)   , val_int(n)      { }
Value::Value(double d)           : type(FLOAT)     , defined(true)   , val_float(d)    { }
Value::Value(float d)            : type(FLOAT)     , defined(true)   , val_float(d)    { }
Value::Value(const String &s)    : type(STRING)    , defined(true)   , val_string(s)   { }
Value::Value(bool b)             : type(BOOL)      , defined(true)   , val_bool(b)     { }
Value::Value(const Clip::Clip& c): type(CLIP)      , defined(true)   , val_clip(c)     { }
Value::Value(const Operator &o)  : type(OPERATOR)  , defined(true)   , val_operator(o) { }
Value::Value(const Value &v)     : type(v.type)          , val_clip(v.val_clip),
                                   val_int(v.val_int)    , val_operator(v.val_operator), 
                                   val_float(v.val_float), val_string(v.val_string),
                                   val_bool(v.val_bool)  , defined(v.defined) { }

int      Value::toInt()       const { return val_int; }
double   Value::toFloat()     const { return val_float; }
bool     Value::toBool()      const { return val_bool; }
String   Value::toString()    const { return val_string; }
Operator Value::toOperator()  const { return val_operator; }
Clip::Clip Value::toClip()    const { return val_clip; }

Value::operator bool()     const { return toBool(); }
Value::operator Operator() const { return toOperator(); }
Value::operator double()   const { return toFloat(); }
Value::operator float()    const { return float(toFloat()); }
Value::operator int()      const { return toInt(); }
Value::operator Clip::Clip() const { return toClip(); }
Value::operator String()   const { return toString(); }
Value::operator Type()     const { return type; }

bool Value::is_defined() const { return defined; }
void Value::define(bool d) { defined = d; }

/* Parameter */
Parameter::Parameter()                                         : value()      , name("")     { }
Parameter::Parameter(Type type)                                : value(type)  , name("")     { }
Parameter::Parameter(const Value &value)                       : value(value) , name("")     { }
Parameter::Parameter(const Value &value, const String &name)   : value(value) , name(name)   { }

Parameter::operator String()  const { return name; }
Parameter::operator Value()   const { return value; }
Parameter::operator Type()    const { return value; }

void Parameter::define(bool d) { value.define(d); }

/* Parameters */
Parameters::Parameters(int nCount) : std::vector<Parameter>(nCount) { }

Value Parameters::operator[](String name) const
{
   for ( const_iterator it = begin(); it != end(); it++ )
      if ( name == String(*it) )
         return *it;

   return Value();
}

Parameter &Parameters::operator[](int n) { return std::vector<Parameter>::operator[](n); }

Signature &addDefaultCommons(Signature &signature)
{
   signature.push_back(Parameter(Value(Operator(PROCESS)), "Y"));
   signature.push_back(Parameter(Value(Operator(NONE)), "U"));
   signature.push_back(Parameter(Value(Operator(NONE)), "V"));
   signature.push_back(Parameter(0     , "offX"));
   signature.push_back(Parameter(0     , "offY"));
   signature.push_back(Parameter(-1    , "w"));
   signature.push_back(Parameter(-1    , "h"));
   signature.push_back(Parameter(true  , "mmx"));
   signature.push_back(Parameter(true  , "isse"));
   signature.push_back(Parameter(true  , "sse2"));
   signature.push_back(Parameter(false , "sse3"));
   signature.push_back(Parameter(true  , "d3now"));
   signature.push_back(Parameter(true  , "d3now2"));

   return signature;
}

} }
