#ifndef __Mt_Params_H__
#define __Mt_Params_H__

#include "../../common/common.h"
#include "../../common/clip/clip.h"

namespace MaskTools { namespace Parameters {

typedef enum {
   INT,
   FLOAT,
   STRING,
   BOOL,
   CLIP,
   OPERATOR,
   UNDEFINED,

} Type;

typedef enum {
   YV12,
   I420,
   YUVP,

   UNSUPPORTED,

} Colorspace;

typedef enum {
   NONE,
   MEMSET,
   COPY,
   PROCESS,
   COPY_SECOND,

} Mode;

class Operator {
   Mode mode;
   Byte nValue;      /* only for mode == memset */
public:
   Operator();
   Operator(Mode mode);
   Operator(Byte nValue);
   Operator(int nValue);
   bool operator==(const Operator &operation) const; /* loose, only checks for mode equality */
   bool operator==(Mode mode) const;
   operator Mode() const;

   int value() const;
};

class Value {

   Type     type;
   bool     defined;

   Clip::Clip val_clip;
   int      val_int;
   String   val_string;
   bool     val_bool;
   double   val_float;
   Operator val_operator;

public:
   Value();
   Value(int n);
   Value(double d);
   Value(float d);
   Value(const String &s);
   Value(bool b);
   Value(const Value &v);
   Value(const Clip::Clip& c);
   Value(const Operator &o);
   Value(Type t);

   operator    int()          const;
   operator    double()       const;
   operator    float()        const;
   operator    bool()         const;
   operator    String()       const;
   operator    Clip::Clip()   const;
   operator    Operator()     const;
   operator    Type()         const;

   int         toInt()        const;
   double      toFloat()      const;
   bool        toBool()       const;
   String      toString()     const;
   Clip::Clip  toClip()       const;
   Operator    toOperator()   const;

   bool is_defined() const;
   void define(bool d);
};

/* parameter will be used to defined a filter signature, but also to contain the actual value of
   a parameter once the filter has been called */
class Parameter {
   Value       value;
   String      name;
public:
   Parameter();
   Parameter(Type type);
   Parameter(const Value &value);
   Parameter(const Value &value, const String &name);
   void define(bool d);
   operator    String()       const;
   operator    Value()        const;
   operator    Type()         const;
};

typedef std::vector<Parameter> Signature;

Signature &addDefaultCommons(Signature &signature);

class Parameters : public std::vector<Parameter>
{
public:
   Value Y, U, V;
   Value nXOffset, nYOffset, nWidth, nHeight;
   Value flags;
   Clip::Clip clip;

   Parameters(int nCount);
   Value operator[](String name) const;
   Parameter &operator[](int n);
};

} }

#endif
