#include "filter.h"
#include "../../../common/parser/parser.h"

using namespace MaskTools;
using namespace MaskTools::Picture;
using namespace MaskTools::Parameters;
using namespace MaskTools::Constraints;
using namespace MaskTools::Parser;

namespace MaskTools { namespace Filters { namespace Lut { namespace Single {

void Filter::ProcessY(PlaneProcessor *processor, Plane &dst, CPlane &src1,
                      CPlane &src2, CPlane &src3)
{
   static_cast<Processor*>(processor)(dst.plane(), dst.pitch(), nWidth, nHeight, yLut);
}

void Filter::ProcessU(PlaneProcessor *processor, Plane &dst, CPlane &src1,
                      CPlane &src2, CPlane &src3)
{
   static_cast<Processor*>(processor)(dst.plane(), dst.pitch(), nWidthUV, nHeightUV, uLut);
}

void Filter::ProcessV(PlaneProcessor *processor, Plane &dst, CPlane &src1,
                      CPlane &src2, CPlane &src3)
{
   static_cast<Processor*>(processor)(dst.plane(), dst.pitch(), nWidthUV, nHeightUV, vLut);
}

Filter::Filter(const Parameters::Parameters &parameters) :
MaskTools::Filter(parameters)
{
   Parser::Parser parser = Parser::getDefaultParser().addSymbol(Symbol::X);

   /* compute the luts */
   /* Y */
   if ( parameters["yExpr"].is_defined() ) parser.parse(parameters["yExpr"].toString(), " ");
   else parser.parse(parameters["expr"].toString(), " ");
   Context yCtx(parser.getExpression());
   if ( !yCtx.check() )
   {
      error = "lut : invalid yExpr";
      return;
   }
   for ( int x = 0; x < 256; x++ )
      yLut[x] = yCtx.setVariable("x", x).compute_byte();

   /* U */
   if ( parameters["uExpr"].is_defined() ) parser.parse(parameters["uExpr"].toString(), " ");
   else parser.parse(parameters["expr"].toString(), " ");
   Context uCtx(parser.getExpression());
   if ( !uCtx.check() )
   {
      error = "lut : invalid uExpr";
      return;
   }
   for ( int x = 0; x < 256; x++ )
      uLut[x] = uCtx.setVariable("x", x).compute_byte();

   /* V */
   if ( parameters["vExpr"].is_defined() ) parser.parse(parameters["vExpr"].toString(), " ");
   else parser.parse(parameters["expr"].toString(), " ");
   Context vCtx(parser.getExpression());
   if ( !vCtx.check() )
   {
      error = "lut : invalid vExpr";
      return;
   }
   for ( int x = 0; x < 256; x++ )
      vLut[x] = vCtx.setVariable("x", x).compute_byte();

   /* add the processor */
   addProcessor(Constraints::Processor<PlaneProcessor>(PPlaneProcessor(lut_c), Constraint(CPU_NONE, 1, 1, 1, 1), 0));
}

String Filter::getName() { return "mt_lut"; }

Signature Filter::getSignature()
{
   Signature signature;
   signature.push_back(Parameter(CLIP, ""));
   signature.push_back(Parameter(String("x"), "expr"));
   signature.push_back(Parameter(String("x"), "yExpr"));
   signature.push_back(Parameter(String("x"), "uExpr"));
   signature.push_back(Parameter(String("x"), "vExpr"));

   return addDefaultCommons(signature);
}

Clip::Inputs Filter::inputClips = Clip::IPOneFrame();

} } } }