#include "filter.h"
#include "../../../common/parser/parser.h"

using namespace MaskTools;
using namespace MaskTools::Picture;
using namespace MaskTools::Parameters;
using namespace MaskTools::Constraints;
using namespace MaskTools::Parser;

namespace MaskTools { namespace Filters { namespace Mask { namespace Mapped {

void Filter::ProcessY(PlaneProcessor *processor, Plane &dst, CPlane &src1,
                      CPlane &src2, CPlane &src3)
{
   static_cast<Processor*>(processor)(dst.plane(), dst.pitch(), src1.plane(), src1.pitch(),
                                      src2.plane(), src2.pitch(), src3.plane(), src3.pitch(),
                                      matrix, nWidth, nHeight);
}

void Filter::ProcessU(PlaneProcessor *processor, Plane &dst, CPlane &src1,
                      CPlane &src2, CPlane &src3)
{
   static_cast<Processor*>(processor)(dst.plane(), dst.pitch(), src1.plane(), src1.pitch(),
                                      src2.plane(), src2.pitch(), src3.plane(), src3.pitch(),
                                      matrix, nWidthUV, nHeightUV);
}

void Filter::ProcessV(PlaneProcessor *processor, Plane &dst, CPlane &src1,
                      CPlane &src2, CPlane &src3)
{
   static_cast<Processor*>(processor)(dst.plane(), dst.pitch(), src1.plane(), src1.pitch(),
                                      src2.plane(), src2.pitch(), src3.plane(), src3.pitch(),
                                      matrix, nWidthUV, nHeightUV);
}


Filter::Filter(const Parameters::Parameters &parameters) : MaskTools::Filter(parameters)
{
   /* add the processors */
   print(LOG_INFO, "Mappedmask : using custom detector");
   std::list<Symbol> coeffs = Parser::getDefaultParser().parse(parameters["mode"].toString(), " ").getExpression();
   memset(matrix, 0, sizeof(matrix));
   for ( int i = 0; i < 9; i++ )
   {
      if ( !coeffs.size() )
      {
         error = "Mappedmask : unvalid custom detector";
         return;
      }
      matrix[9] += abs(matrix[i] = Word(coeffs.front().getValue(0,0,0)));
      coeffs.pop_front();
   }

   if ( coeffs.size() )
      matrix[9] = Word(coeffs.front().getValue(0,0,0));

   if ( !matrix[9] )
      matrix[9] = 1;

   print(LOG_INFO, "Mappedmask : %3i %3i %3i\n"
                   "             %3i %3i %3i\n"
                   "             %3i %3i %3i\n"
                   "normalize  : %3i", matrix[0], matrix[1], matrix[2], matrix[3], matrix[4], matrix[5], matrix[6], matrix[7], matrix[8], matrix[9]);
   addProcessor(Constraints::Processor<PlaneProcessor>(PPlaneProcessor(mask_c), Constraint(CPU_NONE, 1, 1, 1, 1), 0));
}

String Filter::getName() { return "mt_mapped"; }

Signature Filter::getSignature()
{
   Signature signature;
   signature.push_back(Parameter(CLIP, ""));
   signature.push_back(Parameter(CLIP, ""));
   signature.push_back(Parameter(CLIP, ""));
   signature.push_back(Parameter(String("-1 -1 -1 -1 8 -1 -1 -1 -1"), "mode"));

   return addDefaultCommons(signature);
}

Clip::Inputs Filter::inputClips = Clip::OneFrame();

} } } }
