#include "filter.h"
#include "../../../filters/mask/functions.h"

using namespace MaskTools;

static Byte convolution(Byte a11, Byte a21, Byte a31, Byte a12, Byte a22, Byte a32, Byte a13, Byte a23, Byte a33, const Word matrix[10], int nLowThreshold, int nHighThreshold)
{
   return applyThresholds(abs((a11 * matrix[0] + a21 * matrix[1] + a31 * matrix[2] + 
                               a12 * matrix[3] + a22 * matrix[4] + a32 * matrix[5] +
                               a13 * matrix[6] + a23 * matrix[7] + a33 * matrix[8]) / matrix[9]), nLowThreshold, nHighThreshold);
}

class Thresholds {
   const Byte *pMin, *pMax;
   int nMinPitch, nMaxPitch;
public:
   Thresholds(const Byte *pMin, int nMinPitch, const Byte *pMax, int nMaxPitch) :
   pMin(pMin), pMax(pMax), nMinPitch(nMinPitch), nMaxPitch(nMaxPitch)
   {
   }

   int minpitch() const { return nMinPitch; }
   int maxpitch() const { return nMaxPitch; }
   void nextRow() { pMin += nMinPitch; pMax += nMaxPitch; }
   Byte min(int x) const { return pMin[x]; }
   Byte max(int x) const { return pMax[x]; }
};

void MaskTools::Filters::Mask::Mapped::mask_c(Byte *pDst, int nDstPitch, const Byte *pSrc1, int nSrc1Pitch, 
                                              const Byte *pSrc2, int nSrc2Pitch, const Byte *pSrc3, int nSrc3Pitch,
                                              const Word matrix[10], int nWidth, int nHeight)
{
   generic_c<convolution, Thresholds>(pDst, nDstPitch, pSrc1, nSrc1Pitch, Thresholds(pSrc2, nSrc2Pitch, pSrc3, nSrc3Pitch), matrix, nWidth, nHeight);
}