#include "filter.h"

using namespace MaskTools;
using namespace MaskTools::Picture;
using namespace MaskTools::Parameters;
using namespace MaskTools::Constraints;

namespace MaskTools { namespace Filters { namespace Mask { namespace Motion {

void Filter::ProcessY(PlaneProcessor *processor, Plane &dst, CPlane &src1,
                      CPlane &src2, CPlane &src3)
{
   nSceneChange = static_cast<Processor*>(processor)(dst.plane(), dst.pitch(), src1.plane(), src1.pitch(), nLowThresholdY, nHighThresholdY, nMotionThreshold, 0, nSceneChangeValue, nWidth, nHeight) ? 3 : 2;
}

void Filter::ProcessU(PlaneProcessor *processor, Plane &dst, CPlane &src1,
                      CPlane &src2, CPlane &src3)
{
   static_cast<Processor*>(processor)(dst.plane(), dst.pitch(), src1.plane(), src1.pitch(), nLowThresholdUV, nHighThresholdUV, nMotionThreshold, nSceneChange, nSceneChangeValue, nWidthUV, nHeightUV);
}

void Filter::ProcessV(PlaneProcessor *processor, Plane &dst, CPlane &src1,
                      CPlane &src2, CPlane &src3)
{
   static_cast<Processor*>(processor)(dst.plane(), dst.pitch(), src1.plane(), src1.pitch(), nLowThresholdUV, nHighThresholdUV, nMotionThreshold, nSceneChange, nSceneChangeValue, nWidthUV, nHeightUV);
}


Filter::Filter(const Parameters::Parameters &parameters) : MaskTools::Filter(parameters)
{
   nLowThresholdY = clip(parameters["thY1"].toInt(), 0, 255);
   nHighThresholdY = clip(parameters["thY2"].toInt(), 0, 255);
   nLowThresholdUV = clip(parameters["thC1"].toInt(), 0, 255);
   nHighThresholdUV = clip(parameters["thC2"].toInt(), 0, 255);
   nMotionThreshold = clip(parameters["thT"].toInt(), 0, 255);
   nSceneChangeValue = clip(parameters["scvalue"].toInt(), 0, 255);

   /* add the processors */
   addProcessor(Constraints::Processor<PlaneProcessor>(PPlaneProcessor(mask_c), Constraint(CPU_NONE, 1, 1, 1, 1), 0));
   addProcessor(Constraints::Processor<PlaneProcessor>(PPlaneProcessor(mask8_mmx), Constraint(CPU_MMX, 8, 1, 1, 1), 1));
   addProcessor(Constraints::Processor<PlaneProcessor>(PPlaneProcessor(mask8_isse), Constraint(CPU_ISSE, 8, 1, 1, 1), 2));
   addProcessor(Constraints::Processor<PlaneProcessor>(PPlaneProcessor(mask8_3dnow), Constraint(CPU_3DNOW, 8, 1, 1, 1), 3));
}

String Filter::getName() { return "mt_motion"; }

Signature Filter::getSignature()
{
   Signature signature;
   signature.push_back(Parameter(CLIP, ""));
   signature.push_back(Parameter(10, "thY1"));
   signature.push_back(Parameter(10, "thY2"));
   signature.push_back(Parameter(10, "thC1"));
   signature.push_back(Parameter(10, "thC2"));
   signature.push_back(Parameter(10, "thT"));
   signature.push_back(Parameter(0, "sc_value"));

   return addDefaultCommons(signature);
}

Clip::Inputs Filter::inputClips = Clip::IPTemporalOneFrame();

} } } }